﻿<?php
/*
*    Copyright 2008,2009, 2010 Maarch
*
*  This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief Batch script to load autofoldering trees in the database
* (af_node_items table)
*
*
* @file
* @author  Claire Figueras  <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup autofoldering
*/

/**
 * Errors :
 *  1  : Configuration file missing
 *  2  : Mapping file missing
 *  3  : Error with script arguments
 *  4  : Log directory is mandatory
 *  5  : Log directory does not exists or is not writable
 *  6  : Update mode not compatible with from or to
 *  7  : Configuration file does not exist
 *  8  : Mapping file does not exist
 *  9  : Error on loading config file
 *  10 : Mapping file not valid
 *  11 : Error on loading mapping file
 *  12 : Problem with php include path
 *  13 : SQL Query Error
 *  14 : Tree identifier already declared
 *  15 : Datatype unknown
 *  16 : SQL load tree error (bulk load)
 *  17 : Sql directory is mandatory in bulk mode
 */
date_default_timezone_set('Europe/Paris');
echo 'Start : '.date("Y-m-d H:i:s")."\n";

// Class to manage files includes errors
class IncludeFileError extends Exception
{
    public function __construct($file)
    {
        $this->file = $file;
        parent::__construct("Include File \"$file\" is missing!", 1);
    }
}

// File include function which test file existence before inclusion to avoid
// error and throw an IncludeFileError error in case file is missing
function MyInclude($file)
{
    if (file_exists($file)) {
        include_once($file);
    } else {
        throw new IncludeFileError($file);
    }
}

try {
    include 'Maarch_CLITools/ArgsParser.php';
    include 'Maarch_CLITools/Logger.php';
    include 'Maarch_CLITools/FileHandler.php';
    include 'Maarch_CLITools/ConsoleHandler.php';
}
catch(IncludeFileError $e) {
    echo "Maarch_CLITools required ! \n (pear.maarch.org)\n";
    exit();
}

// Define globals variables
$configFile = '';
$insertMode = true;
$updateMode = false;
$trees = array();
$databasetype = '';
$runningDate = date('Y-m-d H:i:s');
$csvSeparator = ';';
$sqlFile = '';
$sqlDir = '.';
$updateFrom = false;
$updateTo = false;

// Make the database query and logs it if necessary
function doQuery($dbConn, $queryTxt, $logger)
{
    $dbConn->connect();
    $res = $dbConn->query($queryTxt, true, true);

    if (!$res) {
        $logger->write('SQL Query error : ' . $queryTxt, 'ERROR', 13);
        if (!$GLOBALS['insertMode']) {
            // fclose($GLOBALS['sqlFile']);
        }
        exit(13);
    }
    $logger->write('SQL query: ' . $queryTxt, 'DEBUG');
    return true;
}

// For given value and datatype, returns string to insert in database or false
// if datatype is unknown
function returnValueToInsertStr($value, $datatype)
{
    $func = new functions();
    if ($datatype == 'STRING') {
        return  "'" . $func->protect_string_db(
            $value, $GLOBALS['databasetype']
        ) . "'";
    } else if ($datatype == 'INTEGER') {
        return $value;
    } else if ($datatype == 'DATE' || $datatype == 'YEAR'
        || $datatype == 'MONTH' || $datatype == 'DAY'
    ) {
        return "'" . $value . "'";
    }
    return false;
}
// Returns sql statement to be used in where clause, depending on the datatype
function returnWhereClausePart($key, $value, $datatype)
{
    $func = new functions();
    if ($datatype == 'STRING') {
        return $key . " = '" . $func->protect_string_db(
            $value, $GLOBALS['databasetype']
        ) . "'";
    } else if ($datatype == 'INTEGER') {
        return $key . ' = ' . $value;
    } else if ($datatype == 'DATE' ) {
        if ($GLOBALS['databasetype'] == "POSTGRESQL") {
            return " cast(" . $key . " as date) = '" . $value . "'";
        } else {
            return  " " . $key . " = '" . $value . "'";
        }
    } else if ($datatype == 'YEAR' || $datatype == 'MONTH'
        || $datatype == 'DAY'
    ) {
        return " " . $key . "  = '" . $value . "'";
    }
    return false;
}

// Define this sripts arguments
function defineScriptArgs()
{
     // Define allowed arguments in the script
    $argsparser = new ArgsParser();
    $argsparser->add_arg(
        'mapping', array(
            'short'     => 'm',
            'long'      => 'mapping',
            'mandatory' => true,
            'help'      => 'Mapping file path is mandatory.',
        )
    );
    $argsparser->add_arg(
        'config', array(
            'short'     => 'c',
            'long'      => 'config',
            'mandatory' => true,
            'help'      => 'Config file path is mandatory.',
        )
    );
    $argsparser->add_arg(
        'dirlog', array(
            'short'     => 'l',
            'long'      => 'dirlog',
            'mandatory' => false,
            'default'   => '.',
            'help'      => 'Directory of the log file, current directory if not'
            .  ' specified',
        )
    );

    $argsparser->add_arg(
        'from', array(
            'short'     => 'f',
            'long'      => 'from',
            'mandatory' => false,
            'help'      => 'Parsing resources table from this res_id number. '
            . 'This mode does not erase the existing tree. '
            . 'Incompatible with --update.',
        )
    );
    $argsparser->add_arg(
        'to', array(
            'short'     => 't',
            'long'      => 'to',
            'mandatory' => false,
            'help'      => 'Parsing resources table to this res_id number. '
            . 'This mode does not erase the existing tree. '
            . 'Incompatible with update mode.',
        )
    );
    /*
    $argsparser->add_switch(
        'update', array(
            'short' => 'u',
            'long' => 'update',
            'help' => 'Update mode, instead of buiding trees from the ground, '
                      . 'we keep existing trees and look only for the new '
                      . 'documents (based on date). Incompatible with --from '
                      . 'or --to. '
        )
    );
    */

    $argsparser->add_switch(
        'bulk', array(
            'short' => 'b',
            'long'  => 'bulk',
            'help'  => 'Bulk mode, instead of classic sql inserts we use a csv '
            . 'file ',
        )
    );

    $argsparser->add_switch(
        'novalidation', array(
            'short' => 'x',
            'long'  => 'novalidation',
            'help'  => 'If activated, do not make the dom validation of '
            . 'the mapping file.',
        )
    );

    $argsparser->add_switch(
        'checkbranches', array(
            'short' => 'n',
            'long'  => 'checkbranches',
            'help'  => 'Allow to check branches existence (sql queries), this'
            . ' option slows down the script and is not necessary if'
            . ' the target view is defined correctly (af_target '
            . 'tables row excluded from the view).',
        )
    );

    $argsparser->add_arg(
        'sqldir', array(
            'short'     => 's',
            'long'      => 'sqldir',
            'mandatory' => false,
            'default'   => '.',
            'help'      => 'Specify the directory where to store the sql file '
            . 'used in bulk mode. (Current directory by default)',
        )
    );
    return $argsparser;
}

// Main function
function main()
{
    // Define script arguments
    $argsparser = defineScriptArgs();

    // Log management init
    $logger = new Logger();
    $logger->set_threshold_level('DEBUG');
    $console = new ConsoleHandler();
    $logger->add_handler($console);

    // Parse script options
    try {
        $options = $argsparser->parse_args($GLOBALS['argv']);
        // If option = help then options = false and the script continues ...
        if ($options == false) {
            exit(0);
        }
    }
    catch(MissingArgumentError $e)
    {
        if ($e->arg_name == 'config') {
            $logger->write('Configuration file missing', 'ERROR', 1);
            exit(1);
        } elseif ($e->arg_name == 'mapping') {
            $logger->write('Mapping file missing', 'ERROR', 2);
            exit(2);
        } else {
            $logger->write('Arguments Error, try -h to view help', 'ERROR', 3);
            exit(3);
        }
    }
    // If no dirlog set, exit
    if (!isset($options['dirlog'])) {
        $logger->write('Log directory is mandatory', 'ERROR', 4);
        exit(4);
    }
    // If dirlog does not exist or script has no sufficient rights, exit
    if (!is_dir($options['dirlog']) || !is_writable($options['dirlog'])) {
        $logger->write(
            'Log directory (' . $options['dirlog'] . ') does not exists or '
            . 'is not writable', 'ERROR', 5
        );
        exit(5);
    }

    if (isset($options['bulk']) && !isset($options['sqldir'])) {
        $logger->write('Sql directory is mandatory in bulk mode', 'ERROR', 17);
        exit(4);
    }

    $file = new FileHandler(
        $options['dirlog'] . DIRECTORY_SEPARATOR . 'log.txt'
    );
    $logger->add_handler($file);

    $txt = '';
    foreach (array_keys($options) as $key) {
        if (isset($options[$key]) && $options[$key] == false) {
            $txt .= $key . '=false,';
        } else {
            $txt .= $key . '=' . $options[$key] . ',';
        }
    }
    $logger->write($txt, 'DEBUG');

    // Assign values to global variables
    // $GLOBALS['updateMode'] = $options['update'];
    $GLOBALS['updateMode'] = true;
    $GLOBALS['insertMode'] = !$options['bulk'];
    $GLOBALS['configFile'] = $options['config'];
    $GLOBALS['checkbranches'] = $options['checkbranches'];
    $GLOBALS['sqlDir'] = $options['sqldir'];

    if (isset($options['from'])) {
        $GLOBALS['updateFrom'] = $options['from'];
        $GLOBALS['updateMode'] = false;
    }
    if (isset($options['to'])) {
        $GLOBALS['updateTo'] = $options['to'];
        $GLOBALS['updateMode'] = false;
    }

      // Check modes compatibilities
    /* if (isset($options['update']) && $options['update']
            && (isset($options['from']) || isset($options['to']))) {
        $logger->write(
            'update option not compatible with from or to options, '
            . 'for more informations see help (-h)', 'ERROR', 6
        );
        exit(6);
    }*/

    $txt = 'Launching autofoldering_load_db ';
    if ($GLOBALS['updateMode'] == true) {
        $txt .= ' in update mode (computes only new documents)';
    } else if ($GLOBALS['updateFrom'] <> false
        ||$GLOBALS['updateTo'] <> false
    ) {
        $txt .= ' for res_ids';
        if ($GLOBALS['updateFrom'] <> false) {
            $txt .= ' starting from ' . $GLOBALS['updateFrom'];
        }
        if ($GLOBALS['updateTo'] <> false) {
            $txt .= ' up to ' . $GLOBALS['updateTo'];
        }
    }
    /*
        else {
            $txt .= ' in normal mode (recomputes all trees)';
        }
    */
    $logger->write($txt, 'INFO');

    // Test existence of config file
    if (!file_exists($GLOBALS['configFile'])) {
        $logger->write(
            'Configuration file ' . $GLOBALS['configFile'] . ' does not exist',
            'ERROR', 7
        );
        exit(7);
    }
    // Test existence of mapping file
    if (!file_exists($options['mapping'])) {
        $logger->write(
            'Mapping file ' . $options['mapping'] . ' does not exist', 'ERROR',
            8
        );
        exit(8);
    }

    // Load config file
    $logger->write('Load xml config file : ' . $GLOBALS['configFile'], 'INFO');

    $xmlconfig = simplexml_load_file($GLOBALS['configFile']);

    if ($xmlconfig == FALSE) {
        $logger->write(
            'Error on loading config file : ' . $GLOBALS['configFile'],
            'ERROR', 9
        );
        exit(9);
    }

    $config = $xmlconfig->CONFIG;
    $lang = (string) $config->lang;
    $maarchDirectory = (string) $config->MaarchDirectory;
    $maarchApps = (string) $config->MaarchApps;
    $maarchApps = (string) $config->AutofolderingUrl;
    $logLevel = (string) $config->LogLevel;
    $displayedLogLevel = (string) $config->DisplayedLogLevel;
    if (isset($config->csvSeparator) && !empty($config->csvSeparator)) {
        $GLOBALS['csvSeparator'] = (string) $config->csvSeparator;
    }
    $GLOBALS['databasetype'] = (string) $xmlconfig->CONFIG_BASE->databasetype;
    $databasename = (string) $xmlconfig->CONFIG_BASE->databasename;
    $databaseserver = (string) $xmlconfig->CONFIG_BASE->databaseserver;
    $databaseserverport = (string) $xmlconfig->CONFIG_BASE->databaseserverport;
    $databaseuser = (string) $xmlconfig->CONFIG_BASE->databaseuser;
    $databasepassword = (string) $xmlconfig->CONFIG_BASE->databasepassword;
    if ($logLevel == 'DEBUG') {
        error_reporting(E_ALL);
    }
    $logger->change_handler_log_level($file, $logLevel);
    $logger->change_handler_log_level($console, $displayedLogLevel);

    unset($xmlconfig);

    if (! isset($options['novalidation'])) {
        // Validate mapping file with dtd
        $logger->write(
            'Begin validation mapping file : ' . $options['mapping'], 'INFO'
        );

        $dom = new DOMDocument();
        $dom->validateOnParse = true;
        $dom->load($options['mapping']);

        if ($dom->validate() == false) {
            $logger->write(
                'Mapping file not valid (see DTD) : ' . $options['mapping'],
                'ERROR', 10
            );
            unset($dom);
            exit(10);
        }

        unset($dom);
        $logger->write(
            'End validation mapping file : '.$options['mapping'], 'INFO'
        );
    }
    // Load mapping file
    $logger->write('Load xml mapping file : '.$options['mapping'], 'INFO');

    $xmlInput = simplexml_load_file($options['mapping']);

    if ($xmlInput == FALSE) {
        $logger->write(
            'Error on loading mapping file : ' . $options['mapping'], 'ERROR',
            11
        );
        exit(11);
    }

    // Include maarch files
    set_include_path(get_include_path() . PATH_SEPARATOR . $maarchDirectory);
    try {
        MyInclude(
            $maarchDirectory . 'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_functions.php'
        );
        MyInclude(
            $maarchDirectory . 'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_db.php'
        );
        MyInclude(
            $maarchDirectory . 'core' . DIRECTORY_SEPARATOR . 'class'
            . DIRECTORY_SEPARATOR . 'class_core_tools.php'
        );
        MyInclude(
            $maarchDirectory . 'modules/autofoldering/class/'
            . 'af_node_items_controler.php'
        );
        MyInclude($maarchDirectory . 'core/core_tables.php');
    } catch(IncludeFileError $e) {
        $logger->write(
            'Problem with the php include path : ' . get_include_path(),
            'ERROR', 12
        );
        exit(12);
    }
    $core = new core_tools();
    $core->load_lang($lang, $maarchDirectory, $maarchApps);

    // Begin parsing mapping file
    $logger->write('Begin parsing mapping file', 'INFO');

    $db = new dbquery($GLOBALS['configFile']);
    $db->connect();

    // Date management in update mode
    /*  if ($GLOBALS['updateMode']) {
        $GLOBALS['last_date'] = NULL;
        $logger->write(
            'Mode update : getting the last date in parameters table', 'INFO'
        );
        $query = 'select param_value_date from ' . PARAM_TABLE
               . " where id = 'af_last_date'";
        doQuery($db, $query, $logger);

        if ($db->nb_result() == 0) {
            // If af_last_date does not exist, insert it in param table
            $query = "insert into " . PARAM_TABLE
                   . " (id, param_value_date) VALUES ('af_last_date', '"
                   . $GLOBALS['runningDate'] . "')";
            doQuery($db, $query, $logger);
        } else {
            // af_last_date does exists
            $q_res = $db->fetch_object();
            $GLOBALS['last_date'] = $q_res->param_value_date;
            $query = "select param_value_date from " . PARAM_TABLE
                   . " where id = 'af_last_last_date'";
            doQuery($db, $query, $logger);

            if ($db->nb_result() == 0) {
                $query = "insert into " . PARAM_TABLE
                      . " (id, param_value_date) VALUES ('af_last_last_date', '"
                       . $GLOBALS['last_date'] . "')";
                doQuery($db, $query, $logger);
            } else {
                $query = "update " . PARAM_TABLE . " set param_value_date = '"
                       . $GLOBALS['last_date']
                       . "' where id = 'af_last_last_date'";
                doQuery($db, $query, $logger);
            }
            $query = "update " . PARAM_TABLE . " set param_value_date = '"
                   . $GLOBALS['runningDate'] . "' where id = 'af_last_date'";
            doQuery($db, $query, $logger);
        }
    }*/

    // Build restriction clause
    $where = '';

    // Based on creation_date
   	/* if (isset($GLOBALS['updateMode']) && $GLOBALS['updateMode'] == true) {
        $where .= " where creation_date < '" . $GLOBALS['runningDate'] . "'";
        if (isset($GLOBALS['last_date'])) {
            $where .= " and creation_date > '" . $GLOBALS['last_date'] . "'";
        }
    }
    // or on res_ids
    else*/
    if ((isset($GLOBALS['updateFrom']) && $GLOBALS['updateFrom'] <> false)
        || (isset($GLOBALS['updateTo']) && $GLOBALS['updateTo'] <> false)
    ) {
        $where .= " where ";
        if (isset($GLOBALS['updateFrom'])
            && $GLOBALS['updateFrom'] <> false
        ) {
            $where .= ' res_id >= ' . $GLOBALS['updateFrom'] . ' and ';
        }
        if (isset($GLOBALS['updateTo']) && $GLOBALS['updateTo'] <> false) {
            $where .= ' res_id <= ' . $GLOBALS['updateTo'] . ' and ';
        }
        $where = preg_replace('/and $/', '', $where);
    }
    // Parse trees
    $db2 = new dbquery($GLOBALS['configFile']);
    $db2->connect();
    $allowedDatatype = array(
        'INTEGER', 'STRING', 'DATE', 'YEAR', 'MONTH', 'DAY',
    );

    foreach ($xmlInput->tree as $tree) {
        $idTree = (string) $tree->id;
        $sourceView = (string) $tree->source_view;

        // Bulk Mode : sql file management
        if (! $GLOBALS['insertMode']) {
            // If sql exists, rename it as old_autofoldering.sql
            if (file_exists(
                $GLOBALS['sqlDir'] . DIRECTORY_SEPARATOR . 'autofoldering_'
                . $idTree . '.sql'
            )
            ) {
                copy(
                    $GLOBALS['sqlDir'] . DIRECTORY_SEPARATOR . 'autofoldering_'
                    . $idTree . '.sql',
                    $GLOBALS['sqlDir'] . DIRECTORY_SEPARATOR
                    . 'old_autofoldering_' . $idTree. '.sql'
                );
                unlink(
                    $GLOBALS['sqlDir'] . DIRECTORY_SEPARATOR . 'autofoldering_'
                    . $idTree . '.sql'
                );
            }
            $GLOBALS['sqlFile'] = fopen(
                $GLOBALS['sqlDir'] . DIRECTORY_SEPARATOR . 'autofoldering_'
                . $idTree . '.sql', 'w'
            );
        }

        $targetTable = (string) $tree->target_table;
        $nodes = $tree->nodes;
        $branchesTotal = 0;
        $newBranchesTotal = 0;
        if (!in_array($idTree, $GLOBALS['trees'])) {
            array_push($GLOBALS['trees'], $idTree);
        } else {
            $logger->write(
                'Tree identifier already declared : ' . $idTree, 'ERROR', 14
            );
            fclose($GLOBALS['sqlFile']);
            exit(14);
        }

        $logger->write('Begin parsing tree node : ' . $idTree, 'INFO');
        $logger->write('Tree source_view : ' . $sourceView, 'DEBUG');
        $logger->write('Tree target_table : ' . $targetTable, 'DEBUG');
        // If not one of update modes, put the tree status to 'DEL'
        // in the af_node_items table to delete later
		/*
        if (!$GLOBALS['updateMode'] && !$GLOBALS['updateFrom']
            && !$GLOBALS['updateTo']) {
            $logger->write(
                'Put status to DEL in database : ' . $idTree, 'INFO'
            );
            $queryTxt = "update " . AF_NODE_ITEMS_TABLE . " set status = 'DEL'"
                       . " where tree_id = '" . $idTree . "'";
            doQuery($db, $queryTxt, $logger);
        }
		*/
        $fields = '';
        $inTargetFields = array();
        $i = 1;
        $nodesDesc = array();
        $idsArr = array();
        $idsFields = '';
        $increment = 1;
        $targetNodeExists = false;
        // Parse tree nodes to get nodes info (to build query)
        foreach ($nodes->node as $node) {
            $datatype = (string) $node->datatype;
            if (!in_array($datatype, $allowedDatatype)) {
                $logger->write(
                    'Datatype for level' . $i . ' unknown: ' . $datatype,
                    'ERROR', 15
                );
                exit(15);
            }

           $idsFields .= ',' . 'level' . $i . '_id';

            $nodesDesc['level'.$i] = array(
                'datatype' => $datatype,
                'in_target_table' => false,
            );
            if (isset($node->in_target_table)) {
                $nodesDesc['level' . $i]['in_target_table'] = true;
                $inTargetFields[] = 'level' . $i ;
                $targetNodeExists = true;
            }
            $i++;
        }
        $arrNodes = array_keys($nodesDesc);
        $fields = implode(',', $arrNodes);
        //main query to get tree branches
        $query = 'select distinct ' . $fields . ' from ' . $sourceView . ' '
               . $where . ' order by ' . $fields;
        doQuery($db, $query, $logger);
        while ($res = $db->fetch_object()) {
            $logger->write('Build where clause ', 'DEBUG');
            // Store in array the where clause parts for each node
            $whereItems = array();
            foreach (array_keys($nodesDesc) as $keyNode) {
                $whereItems[$keyNode] = returnWhereClausePart(
                    $keyNode, $res->$keyNode,
                    $nodesDesc[$keyNode]['datatype']
                );
            }
            $newBranch = true;
            if ($GLOBALS['checkbranches']) {
                // Test existence of branch
                $logger->write('Test existence of branch ', 'DEBUG');
                $newBranch = true;
                $where2 = ' where ' . implode(' and ', $whereItems);

                $query2 = "select level1 from " . $targetTable . " " . $where2;
                doQuery($db2, $query2, $logger);
                if ($db2->nb_result() > 0) {
                    $newBranch = false;
                    $logger->write('Branch exists already ', 'INFO');
                }
            }

            $branchesTotal ++;
            if ($newBranch) {
                $newBranchesTotal ++;
                if ($newBranchesTotal % 10 == 0) {
                    $logger->write(
                        'Processing branch '.$newBranchesTotal, 'INFO'
                    );
                }
                if ($targetNodeExists) {
                    $specificLabels = array();
                    $logger->write(
                        'Some labels must be in target_table ', 'DEBUG'
                    );

                    $logger->write(
                        'Compute where clause to get those labels ', 'DEBUG'
                    );

                    for ($i = 0; $i < count($inTargetFields); $i++) {
                        $logger->write(
                            'Get label for ' . $inTargetFields[$i] . ' ',
                            'DEBUG'
                        );
                        $tmpArr = $arrNodes;
                        $tmpArr2 = array();
                        while (1) {
                            $val = array_shift($tmpArr);
                            $tmpArr2 [] = $val;
                            if ($val == $inTargetFields[$i]) {
                                break;
                            }
                        }
                        $fieldsTmp = implode(',', $tmpArr2);
                        $where2 = ' where ';
                        for ($j = 0; $j < count($tmpArr2); $j++) {
                            $where2 .= $whereItems[$tmpArr2[$j]] . " and ";
                        }
                        $where2 = preg_replace('/and $/', '', $where2);
						
						$queryTxt = $db2->limit_select(0, 1, $inTargetFields[$i] . "_label as label", $sourceView, $where2);
						
                        doQuery($db2, $queryTxt, $logger);
                        $res2 = $db2->fetch_object();
                        $specificLabels[$inTargetFields[$i]] = $res2->label;
                        $logger->write(
                            'Label value for ' . $inTargetFields[$i] . ' : '
                            . $res2->label, 'DEBUG'
                        );
                    }
                }
                if ($GLOBALS['insertMode']) {
                    $insertValues = '';
                    $moreFields = '';
                    $moreValues = '';
                    foreach (array_keys($nodesDesc) as $keyNode) {
                        $insertValue = returnValueToInsertStr(
                            $res->$keyNode, $nodesDesc[$keyNode]['datatype']
                        );
                        if ($insertValue == false) {
                            $logger->write(
                                'Datatype for ' . $keyNode . ' unknown: '
                                . $nodesDesc[$keyNode]['datatype'], 'ERROR',
                                14
                            );
                            exit(14);
                        }
                        $insertValues .= $insertValue . ',';
                        if (isset($idsArr[$insertValue])
                            && ! empty($idsArr[$insertValue])
                        ) {
                            $moreValues .= ', ' . $idsArr[$insertValue] ;
                        } else {
                            $increment ++;
                            $moreValues .= ', ' . $increment ;
                            $idsArr[$insertValue] = $increment;
                        }
                    }
                    $insertValues = preg_replace('/,$/', '', $insertValues);

                    if ($targetNodeExists) {
                        foreach (array_keys($specificLabels) as $key) {
                            $moreFields .= ',' . $key . '_label';
                            $insertValue = returnValueToInsertStr(
                                $specificLabels[$key],
                                $nodesDesc[$key]['datatype']
                            );
                            $insertValues .= ',' . $insertValue;
                        }
                    }
                    $insertFields = $fields . $moreFields . $idsFields ;
                    $logger->write('Add branch ', 'INFO');
                    $logger->write('Fields : ' . $insertFields, 'DEBUG');
                    $logger->write('Values : ' . $insertValues, 'DEBUG');

                    $query = 'insert into ' . $targetTable . ' ('
                           . $insertFields . ') values (' . $insertValues
                           . $moreValues . ')';
                    doQuery($db2, $query, $logger);
                } else {
                    // Or write a line in the csv file (autofoldering.sql)
                    $csvLine = '';
                    $idsValues = '';
                    foreach (array_keys($nodesDesc) as $keyNode) {
                        $csvLine .= $res->$keyNode
                                  . $GLOBALS['csvSeparator'];
                        if (isset($idsArr[$res->$keyNode])
                            && ! empty($idsArr[$res->$keyNode])
                        ) {
                            $idsValues .= $idsArr[$res->$keyNode]
                                . $GLOBALS['csvSeparator'];
                        } else {
                            $increment ++;
                            $idsValues .= $increment
                                . $GLOBALS['csvSeparator'];
                            $idsArr[$res->$keyNode] = $increment;
                        }
                    }
                    $csvLine = $csvLine . $idsValues;
                    $csvLine = substr_replace(
                        $csvLine, '', strrpos(
                            $csvLine, $GLOBALS['csvSeparator']
                        )
                    );
                    if ($targetNodeExists ) {
                        foreach (array_keys($specificLabels) as $key) {
                            $csvLine .= $GLOBALS['csvSeparator']
                                      . $specificLabels[$key] ;
                        }
                    }
                    $logger->write('CSV line : ' . $csvLine, 'DEBUG');
                    $csvLine .= "\n";
                    fwrite($GLOBALS['sqlFile'], $csvLine);
                    $logger->write('CSV line wrote', 'DEBUG');
                }
            }
        }

        //If not one of update modes, delete the old tree
		/*
        if (!$GLOBALS['updateMode'] && !$GLOBALS['updateFrom']
            && !$GLOBALS['updateTo']) {
            $queryTxt = "delete from " . AF_NODE_ITEMS_TABLE
                       . " where tree_id = '" . $idTree
                       . "' and status = 'DEL'";
            doQuery($db, $queryTxt, $logger);
        }
		*/

        if (! $GLOBALS['insertMode']) {
            fclose($GLOBALS['sqlFile']);
            $pathSql = $GLOBALS['sqlDir'] . DIRECTORY_SEPARATOR
                      . 'autofoldering_' . $idTree . '.sql';
            if (DIRECTORY_SEPARATOR == '\\') {
                $pathSql = str_replace('\\', '\\\\', $pathSql);
            }
            $moreFields = '';
            if ($targetNodeExists ) {
                for ($i = 0; $i < count($inTargetFields); $i++) {
                    $moreFields .= ',' . $inTargetFields[$i] . '_label';
                }
            }
            $fields = $fields . $idsFields . $moreFields;
            $query = '';

            if ($GLOBALS['databasetype'] == 'MYSQL') {
                $query = "LOAD DATA LOCAL INFILE '" . $pathSql
                       . "' INTO TABLE " . $targetTable
                       . " FIELDS TERMINATED BY '" . $GLOBALS['csvSeparator']
                       . "' LINES TERMINATED BY '\\n' (" . $fields . ");";
            } else if ($GLOBALS['databasetype'] == 'POSTGRESQL') {
                $query = "COPY " . $targetTable . " (" . $fields . ") FROM E'"
                       . $pathSql . "' WITH DELIMITER AS '"
                       . $GLOBALS['csvSeparator'] . "';";
            } else if ($GLOBALS['databasetype'] == 'SQLSERVER') {
                $query = "bulk insert " . $databasename . ".dbo."
                       . $targetTable . " from '" . $pathSql . "'
                                with(
                                    DATAFILETYPE = 'widechar',
                                    formatfile='autofoldering_format.xml'
                                );";
                #TO DO test sqlserver avec creation du fichier de format
            } else if ($GLOBALS['databasetype'] == 'ORACLE') {
                if (DIRECTORY_SEPARATOR == '/') {
                    $osName = 'UNIX';
                } else {
                    $osName = 'WINDOWS';
                }

                $query = "LOAD DATA INFILE '" . $pathSql
                       . "' APPEND INTO TABLE " . $targetTable
                       . " FIELDS TERMINATED BY '" . $GLOBALS['csvSeparator']
                       . "' (" . $fields . ")";
                $controlFile = 'control.txt';
                $inF = fopen($controlFile, 'w');
                fwrite($inF, $query);
                fclose($inF);
                if (isset($osName) && $osName == 'UNIX') {
                    $oracleQueryResult = exec(
                        "sqlldr userid=" . $databaseuser . "/"
                        . $databasepassword . " control=control.txt "
                        . "log=logs/log_oracle.txt parallel=true "
                        . "bad=logs/bad_oracle.txt "
                        . "discard=logs/discard_oracle.txt errors=0",
                        $returnTab, $oracleReturn
                    );
                } else {
                    $oracleQueryResult = exec(
                        "sqlldr " . $databaseuser . "/" . $databasepassword
                        . "@" . $databasename . " control=control.txt "
                        . "log=logs/log_oracle.txt parallel=true "
                        . "bad=logs/bad_oracle.txt "
                        . "discard=logs/discard_oracle.txt errors=0",
                        $returnTab, $oracleReturn
                    );
                }
            }
            $logger->write('Load data sql query : ' . $query, 'INFO');

            if ($GLOBALS['databasetype'] <> "ORACLE") {
                doQuery($db, $query, $logger);
            } else if ($oracleReturn <> 0) {
                $logger->write('SQL load tree error : ' . $query, 'ERROR', 16);
                fclose($GLOBALS['sqlFile']);
                exit(16);
            }
        }
        $logger->write('End parsing tree node : ' . $idTree, 'INFO');
        $logger->write('Number of branches found : ' . $branchesTotal, 'INFO');
        $logger->write(
            'Number of new branches created: ' . $newBranchesTotal, 'INFO'
        );
    }
    $logger->write('End parsing mapping file', 'INFO');
    $logger->write('End Autofoldering OK', 'INFO');
}

main();
echo 'End : ' . date("Y-m-d H:i:s") . "\n";
