<?php
/*
*    Copyright 2008,2009,2010 Maarch
*
*  This file is part of Maarch Framework.
*
*   Maarch Framework is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Maarch Framework is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*    along with Maarch Framework.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @brief  Contains the controler of the autofoldering_tree object
* (create, save, modify, etc...)
*
*
* @file
* @author Claire Figueras <dev@maarch.org>
* @date $date$
* @version $Revision$
* @ingroup autofoldering
*/
try {
    require_once 'core/class/class_functions.php';
    require_once 'core/class/class_db.php';
    require_once 'modules/autofoldering/class/autofoldering_tree.php';
    require_once 'modules/autofoldering/class/af_node_items_controler.php';
} catch (Exception $e){
    echo $e->getMessage().' // ';
}

/**
* @brief  Controler of the autofoldering_tree object
*
*<ul>
*  <li>Get an autofoldering_tree object from an id</li>
*</ul>
* @ingroup autofoldering
*/
class autofoldering_tree_controler
{
    /**
    * Returns an autofoldering tree complete with all its nodes
    *
    * @param  $treeId string  Tree identifier
    * @param  $xmlFile string  Path to the autofoldering mapping file
    * @param  $lang string  Language (fr by default)
    * @return aufoldering_tree object with properties from the database or null
    */
    public function get($treeId, $xmlFile, $lang='fr')
    {
        if (! file_exists($xmlFile)) {
            return null;
        }
        if (! isset($treeId) || empty($treeId)) {
            return null;
        }

        $xml = simplexml_load_file($xmlFile);
        foreach ($xml->tree as $tree) {
            $id = (string) $tree->id;
            if ($id == $treeId) {
                $afTree = new autofoldering_tree();
                $afTree->tree_id = $id;
                $afTree->tree_label = (string) $tree->desc;
                $afTree->coll_id = (string) $tree->coll_id;
                $afTree->source_view = (string) $tree->source_view;
                $afTree->target_table = (string) $tree->target_table;
                $afTree->target_view = (string) $tree->target_view;
                $afTree->nodes = array();
                $tmp = array();
                $nodes = $tree->nodes;
                foreach ($nodes->node as $node) {
                    array_push($tmp, self::_loadNode($afTree->tree_id, $node));
                }
                $afTree->nodes = $tmp;

                return $afTree;
            }
        }
        return null;
    }

    /**
    * Returns all autofoldering trees complete with all its nodes
    *
    * @param  $xmlFile string  Path to the autofoldering mapping file
    * @param  $lang string  Language (fr by default)
    * @return aufoldering_tree object with properties from the database or null
    */
    public function get_all_trees($xmlFile, $lang = 'fr')
    {
        if (! file_exists($xmlFile)) {
            return null;
        }
        $xml = simplexml_load_file($xmlFile);

        $trees = array();
        foreach ($xml->tree as $tree) {
            $afTree = new autofoldering_tree();
            $afTree->tree_id = (string) $tree->id;
            $afTree->tree_label = (string) $tree->desc;
            if (!empty($afTree->tree_label) && defined($afTree->tree_label)
             && constant($afTree->tree_label) <> NULL
            ) {
            	$afTree->tree_label = constant($afTree->tree_label);
            }
            $afTree->coll_id = (string) $tree->coll_id;
            $afTree->source_view = (string) $tree->source_view;
            $afTree->target_table = (string) $tree->target_table;
            $afTree->target_view = (string) $tree->target_view;
            $tmp = array();
            $nodes = $tree->nodes;
            foreach ($nodes->node as $node) {
                array_push($tmp, self::_loadNode($afTree->tree_id, $node));
            }
            $afTree->nodes = $tmp;
            $trees[] = $afTree;
        }
        return $trees;
    }

    /**
    * Load a node of the given autofoldering tree in an array
    *
    * @param  $treeId string  Tree identifier
    * @param  $currentNode string  Current node
    * @param  $lang string  Language (fr by default)
    * @return Array of the node
    */
    private function _loadNode($treeId, $currentNode, $lang='fr')
    {
        $func = new functions();
        $pathLang = 'modules' . DIRECTORY_SEPARATOR . 'autofoldering'
                  . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $lang
                  . '.php';
        $comment = (string) $currentNode->comment;
        if (!empty($comment) && defined($comment) 
        	&& constant($comment) <> NULL
        ) {
        	$comment = constant($comment);
        }
        $targetColumn = (string) $currentNode->target_column;
        $datatype = (string) $currentNode->datatype;
        $inTargetTable = false;
        if (isset($currentNode->in_target_table)) {
            $inTargetTable = true;
        }
        $datatype = (string) $currentNode->datatype;
        $order = 'asc';
        if (isset($currentNode->orderby)) {
            $order = (string) $currentNode->orderby;
        }

        if (isset($currentNode->script) && ! empty($currentNode->script)) {
            $script = (string) $currentNode->script;

        } else {
            $script = 'none';
        }
        $nodeArr = array(
        	'comment' => $comment,
        	'target_column' => $targetColumn,
        	'datatype' => $datatype,
        	'in_target_table' => $inTargetTable,
        	'order' => $order,
        	'script' => $script,
        );

        return $nodeArr;
    }

    private function _returnWhereClausePart($key, $value, $datatype)
    {
        $func = new functions();
        $value = $func->protect_string_db($value);

        if ($datatype == 'INTEGER') {
            return $key . " = " . $value;
        } else if ($datatype == 'STRING') {
            return $key . " = '" . $value . "'";
        } else if ($datatype == 'DATE') {
            return $key . "  = '" . $value . "'";
        } else if ($datatype == 'YEAR' || $datatype == 'MONTH'
            || $datatype == 'DAY'
        ) {
            return $key . "  = '" . $value . "'";
        }
        return false;
    }
    /**
    * Creates a security where clause for a node ( with parents nodes security)
    *
    * @param  $treeArr array Tree
    * @param  $node_id string  Node identifier
    * @param  $parents array  Parent nodes of the given node
    * @param  $children array  Children nodes of the given node
    * @return Integer index of the node in the tree
    */
    public function create_where_clause($treeArr, $level, $value,
    $parents=array(), $children=array())
    {
       //  print_r($treeArr);
        $where = '';
        $notWhere = '';
        $db = new dbquery();
        $db->connect();
        $realValue = $this->_getRealValue($db, $level, $value, $treeArr);

        $maxLevel = count($treeArr['nodes']);
        $where = "";
        $where .= self::_returnWhereClausePart(
        	'level' . $level, $realValue, $treeArr['nodes'][$level - 1]['datatype']
        );
        $where = preg_replace(
        	'/level' . $level . ' /',
            $treeArr['nodes'][$level - 1]['target_column'],
            $where
        );

        foreach (array_keys($parents) as $parentLevel) {
            $parentValue = $this->_getRealValue(
                $db, $parentLevel, $parents[$parentLevel], $treeArr
            );
            $where .= " and ".self::_returnWhereClausePart(
            	'level' . $parentLevel, $parentValue,
                $treeArr['nodes'][$parentLevel - 1]['datatype']
            );
            $where = preg_replace(
            	'/level' . $parentLevel . ' /',
                $treeArr['nodes'][$parentLevel - 1]['target_column'],
                $where
            );
        }

        $notIn = '';
        if ($level < $maxLevel) {
            $nextLevel = $level + 1;
            $notIn = " and " . $treeArr['nodes'][$nextLevel - 1]['target_column']
                   . " not in (select distinct level" . $nextLevel
                   . " from " . $treeArr['target_table'] . " " . $where . ")";
        }
        return $where . ' ' . $notIn;
    }

    private function _getRealValue($db, $level, $value, $treeArr)
    {
        $db->connect();
        $db->query(
        	"select distinct level" . $level . " as realvalue from " . $treeArr['target_table']
            . " where level" . $level . "_id = ".$value
        );
        $res = $db->fetch_object();
        if (isset($res->realvalue)) {
            return $res->realvalue;
        }
        return $value;
    }
}